# - Find TinyXML
# Find the native TinyXML includes and library
#
#   TINYXML_FOUND        - True if TinyXML found.
#   TINYXML_INCLUDE_DIRS - where to find tinyxml.h, etc.
#   TINYXML_LIBRARIES    - List of libraries when using TinyXML.
#

IF( TINYXML_INCLUDE_DIR )
    # Already in cache, be silent
    SET( TinyXML_FIND_QUIETLY TRUE )
ENDIF( TINYXML_INCLUDE_DIR )

FIND_PATH( TINYXML_INCLUDE_DIR tinyxml.h
           PATH_SUFFIXES tinyxml )

FIND_LIBRARY( TINYXML_LIBRARY
              NAMES tinyxml
              PATH_SUFFIXES tinyxml )

# handle the QUIETLY and REQUIRED arguments and set TINYXML_FOUND to TRUE if
# all listed variables are TRUE
INCLUDE( FindPackageHandleStandardArgs )
FIND_PACKAGE_HANDLE_STANDARD_ARGS( TinyXML DEFAULT_MSG TINYXML_LIBRARY TINYXML_INCLUDE_DIR )

MARK_AS_ADVANCED( TINYXML_INCLUDE_DIR TINYXML_LIBRARY )

IF(TINYXML_FOUND)

  SET(TINYXML_INCLUDE_DIRS ${TINYXML_INCLUDE_DIR})
  SET(TINYXML_LIBRARIES    ${TINYXML_LIBRARY})

ELSE(TINYXML_FOUND)

  SET(TINYXML_INCLUDE_DIRS)
  SET(TINYXML_LIBRARIES)

ENDIF(TINYXML_FOUND)
